==> heptapod-rails-0.30.1.tar.gz: heptapod-rails-0.30.1/lib/gitlab/mercurial.rb <==
# frozen_string_literal: true

module Gitlab
  module Mercurial
    NULL_NODE = ('00' * 20).freeze

    SHORT_NODE_LAST_CHAR = 11 # [0..SHORT_NODE_LAST_CHAR] makes a length 12

    BookmarksUnsupportedError = Class.new(StandardError)

    class << self
      # Application-wide HGRC files, in a suitable form for the HGRCPATH
      # environment variable.
      def hgrc_path
        Gitlab.config.mercurial.hgrc.join(":")
      end

      def parse_gitlab_branch_for_hg(gitlab_branch)
        if gitlab_branch.start_with?('branch/')
          [gitlab_branch[7..], nil]
        elsif gitlab_branch.start_with?('topic/')
          branch, _slash, topic = gitlab_branch[6..].rpartition('/')
          [branch, topic]
        else
          raise BookmarksUnsupportedError, gitlab_branch
        end
      end

      # true if the given revision is a full hexadecimal changeset hash
      def full_hex_hash?(revision)
        revision =~ /\A\h{40}\z/
      end

      # Provide branchmap key for named branch and topic
      # Branchmap keys are displayed when using `hg heads`, and can
      # be used directly for some operations, notably to address remote
      # revisions.
      def branchmap_key(hg_branch, topic)
        if topic.nil?
          hg_branch
        else
          "#{hg_branch}:#{topic}"
        end
      end

      # Convert GitLab branch to corresponding branchmap key
      #
      # Branchmap keys are displayed when using `hg heads`, and can
      # be used directly for some operations, notably to address remote
      # revisions.
      def branchmap_key_for_gitlab_branch(gitlab_branch)
        self.branchmap_key(*self.parse_gitlab_branch_for_hg(gitlab_branch))
      end

      def parse_hg_archival(lines)
        arch = {}
        lines.each do |l|
          key, value = l.split(":", 2)
          arch[key.to_sym] = value.strip
        end
        arch
      end

      # dir_path: where `.hg_archival.txt` is expected to be
      #           the best is to provide a Pathname instance, but strings are
      #           accepted
      # returns nil if archival file does not exist, otherwise a Hash with
      #         symbol keys.
      def read_hg_archival(dir_path)
        path = Pathname(dir_path).join(".hg_archival.txt")
        return unless path.exist?

        parse_hg_archival(path.readlines)
      end

      # Return the path to expected Mercurial bundles from a Git bundle path
      #
      # As with the case of the repository path, for uniformity and simplicity
      # of high-level callers working with bundles, the Heptapod convention
      # is that bundle paths argument values are what they would be for
      # a Gitlab::Git::Repository, even if there is on disk no Git repository
      # involved.
      #
      # This method returns the actual path to the Mercurial bundle specified
      # by the Git bundle path.
      def hg_bundle_path(git_bundle_path)
        git_bundle_path.sub(/\.bundle$/, '.hg.bundle')
      end

      # Same logic as `hg_bundle_path`, for the additional backup tarball.
      #
      # Since in GitLab, restoration has to be made with the exact same
      # version that was used to product the backup,
      # we are at liberty to change this path easily.
      def hg_backup_additional_path(git_bundle_path)
        git_bundle_path.sub(/\.bundle$/, '.hg.config.tar')
      end

      # Given a theoretical Git bundle path (see hg_bundle_path for context),
      # return `true` if there are Mercurial backup files to restore
      def hg_backup_exists?(git_bundle_path)
        (File.exist?(hg_bundle_path(git_bundle_path)) ||
         File.exist?(hg_backup_additional_path(git_bundle_path))
        )
      end
    end
  end
end

==> heptapod-rails-0.36.2.tar.gz: heptapod-rails-0.36.2/lib/gitlab/mercurial.rb <==
# frozen_string_literal: true

module Gitlab
  module Mercurial
    NULL_NODE = ('00' * 20).freeze

    SHORT_NODE_LAST_CHAR = 11 # [0..SHORT_NODE_LAST_CHAR] makes a length 12

    BookmarksUnsupportedError = Class.new(StandardError)
    # In general, GitLab expects `commit_files` to raise Index::IndexError,
    # so let's subclass it.
    ContentActionError = Class.new(Gitlab::Git::Index::IndexError)
    HgError = Class.new(StandardError)
    InvalidHgrcSetting = Class.new(StandardError)
    MergeError = Class.new(StandardError)
    ObsoleteError = Class.new(StandardError)
    RevisionError = Class.new(StandardError)

    class << self
      # Application-wide HGRC files, in a suitable form for the HGRCPATH
      # environment variable.
      def hgrc_path
        Gitlab.config.mercurial.hgrc.join(":")
      end

      def parse_gitlab_branch_for_hg(gitlab_branch)
        if gitlab_branch.start_with?('branch/')
          [gitlab_branch[7..], nil]
        elsif gitlab_branch.start_with?('topic/')
          branch, _slash, topic = gitlab_branch[6..].rpartition('/')
          [branch, topic]
        else
          raise BookmarksUnsupportedError, gitlab_branch
        end
      end

      # true if the given revision is a full hexadecimal changeset hash
      def full_hex_hash?(revision)
        revision =~ /\A\h{40}\z/
      end

      # Provide branchmap key for named branch and topic
      # Branchmap keys are displayed when using `hg heads`, and can
      # be used directly for some operations, notably to address remote
      # revisions.
      def branchmap_key(hg_branch, topic)
        if topic.nil?
          hg_branch
        else
          "#{hg_branch}//#{topic}"
        end
      end

      # Convert GitLab branch to corresponding branchmap key
      #
      # Branchmap keys are displayed when using `hg heads`, and can
      # be used directly for some operations, notably to address remote
      # revisions.
      def branchmap_key_for_gitlab_branch(gitlab_branch)
        branchmap_key(*parse_gitlab_branch_for_hg(gitlab_branch))
      end

      def parse_hg_archival(lines)
        arch = {}
        lines.each do |l|
          key, value = l.split(":", 2)
          arch[key.to_sym] = value.strip
        end
        arch
      end

      # dir_path: where `.hg_archival.txt` is expected to be
      #           the best is to provide a Pathname instance, but strings are
      #           accepted
      # returns nil if archival file does not exist, otherwise a Hash with
      #         symbol keys.
      def read_hg_archival(dir_path)
        path = Pathname(dir_path).join(".hg_archival.txt")
        return unless path.exist?

        parse_hg_archival(path.readlines)
      end

      # Return the path to expected Mercurial bundles from a Git bundle path
      #
      # As with the case of the repository path, for uniformity and simplicity
      # of high-level callers working with bundles, the Heptapod convention
      # is that bundle paths argument values are what they would be for
      # a Gitlab::Git::Repository, even if there is on disk no Git repository
      # involved.
      #
      # This method returns the actual path to the Mercurial bundle specified
      # by the Git bundle path.
      def hg_bundle_path(git_bundle_path)
        git_bundle_path.sub(/\.bundle$/, '.hg.bundle')
      end

      # Same logic as `hg_bundle_path`, for the additional backup tarball.
      #
      # Since in GitLab, restoration has to be made with the exact same
      # version that was used to product the backup,
      # we are at liberty to change this path easily.
      def hg_backup_additional_path(git_bundle_path)
        git_bundle_path.sub(/\.bundle$/, '.hg.config.tar')
      end

      # Given a theoretical Git bundle path (see hg_bundle_path for context),
      # return `true` if there are Mercurial backup files to restore
      def hg_backup_exists?(git_bundle_path)
        (File.exist?(hg_bundle_path(git_bundle_path)) ||
         File.exist?(hg_backup_additional_path(git_bundle_path))
        )
      end
    end
  end
end
